@@ -2212,6 +2212,11 @@ int test_quicklistCompressAndDecompressQuicklistListpackNode(int argc, char **ar
 
     if (!(flags & UNIT_TEST_LARGE_MEMORY)) return 0;
 
+#ifdef VALKEY_ADDRESS_SANITIZER
+    /* Skip this test under sanitizers to avoid OOM in github actions */
+    return 0;
+#endif
+
     quicklistNode *node = quicklistCreateNode();
     node->entry = lpNew(0);
 
